#!/usr/bin/php
<?php

function read_pidfile($path = '/opt/monitor/var/nagios.lock')
{
	if (!file_exists($path))
		return false;

	if (!($buf = file_get_contents($path)))
		return false;

	return intval($buf);
}

function get_nebmods($path = '/opt/monitor/etc/nagios.cfg')
{
	$mods = array();
	$buf = file_get_contents($path);
	$line = explode("\n", $buf);
	foreach ($line as $l) {
		trim($l);
		if (!strlen($l))
			continue;
		if (substr($l, 0, 1) === '#')
			continue;
		if (strpos($l, 'broker_module') !== 0)
			continue;
		$l = explode(' ', substr($l, 14));
		$mods[] = $l[0];
	}
	return $mods;
}

function copy_modules($output)
{
	$mods = get_nebmods();
	$cp = 0;

	foreach ($output as $oline)
	{
		if (strpos($oline, "/nebmod") === false)
			continue;

		$path = explode(': ', $oline);
		$path = $path[0];
		$mod = array_pop($mods);
		echo "Copying $mod to $path\n";
		copy($mod, $path);
		$cp++;
	}
	return $cp;
}

function status_log_age($path = '/opt/monitor/var/status.log')
{
	if (!file_exists($path))
		return false;

	if (!($st = stat($path)))
		return false;

	return time() - $st['mtime'];
}


function create_debug_report($path = false, $core = false)
{
	global $monitor_pid;

	if (!$path)
		$path = '/tmp/monitor-debug.' . date("Y-m-d_H:i") . ".$monitor_pid";
	echo "report path: $path\n";
	$var_to_check = array
		("event_list_low", "event_list_low_tail",
		 "event_list_high", "event_list_high_tail");
	$gdb_command = "echo y | gdb -batch ";
	if (!$core) {
		$gdb_command .= " --pid=$monitor_pid";
	} else {
		$gdb_command .= " --exec=/opt/monitor/bin/monitor --core=$core";
	}
	$gdb_command .= " -ex 'echo ### Backtrace:\n' -ex 'thread apply all bt full' " .
		"-ex 'echo ### Backtrace end\n'";
	foreach ($var_to_check as $v) {
		$gdb_command .= " -ex 'echo $v' -ex 'p $v'";
	}
	$gdb_command .= " -ex 'echo gdb session end\n' -ex quit 2>&1";
	exec($gdb_command, $output, $result);
	if (copy_modules($output))
		exec($gdb_command, $output, $result);

	array_pop($output);
	$fh = fopen($path, "w");
	$output = join("\n", $output);
	file_put_contents($path, $output . "\n");
	fclose($fh);
}

function restart_monitor()
{
	exec("/bin/sh /etc/init.d/monitor restart");
}

$monitor_pid = read_pidfile();

$report_path = "/dev/stdout";
$autorestart = $core = false;
for ($i = 1; $i < $argc; $i++) {
	switch ($argv[$i]) {
	 case '--autorestart':
		$autorestart = true;
		break;
	 case '--stash':
		$report_path = false;
		break;
	 default:
		$core = $argv[$i];
		if (!is_readable($core)) {
			echo "core file $core isn't readable\n";
			exit(1);
		}
		break;
	}
}

if (!$core) {
	if (!$monitor_pid) {
		echo "Monitor isn't running, and no core-file to examine\n";
		exit(0);
	}

	$age = status_log_age();
	if ($age === false) {
		if (!posix_kill($monitor_pid, 0)) {
			echo "Monitor has crashed\n";
		} else {
			echo "Some process is running with pid $monitor_pid, but there's no status file\n";
		}

		if ($autorestart)
			restart_monitor();
		exit(0);
	}

	echo "Status log is $age seconds old. Monitor ";
	if ($age < 60) {
		echo "is working as intended.\n";
		exit(0);
	} else {
		echo "seems flakey. Creating debug report\n";
	}
}

create_debug_report($report_path, $core);
